﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using BioLab.Biometrics.Face;
using BioLab.Classification.Supervised;
using BioLab.Common;
using System.IO;
using BioLab.ImageProcessing;
using BioLab.GUI.Forms;
using BioLab.GUI.DataViewers;
using System.Globalization;

namespace FaceDetection
{
  public partial class FormFaceDetection : Form
  {
    private ClassifierFaceDetector detector;
    private bool drawResult = false;

    public FormFaceDetection()
    {
      InitializeComponent();
    }

    private void UpdateParams()
    {
      if (detector != null)
      {
        if (textBoxMinWindowSize.Text != "")
        {
          detector.MinimumWindowSize = Convert.ToInt32(textBoxMinWindowSize.Text);
        }
        else
        {
          textBoxMinWindowSize.Text = detector.MinimumWindowSize.ToString();
        }
        if (textBoxMaxScaleFactor.Text != "")
        {
          detector.MaximumScaleFactor = Convert.ToDouble(textBoxMaxScaleFactor.Text);
        }
        else
        {
          textBoxMaxScaleFactor.Text = detector.MaximumScaleFactor.ToString();
        }
        if (textBoxScaleIncStep.Text != "")
        {
          detector.ScaleIncrementStep = Convert.ToDouble(textBoxScaleIncStep.Text);
        }
        else
        {
          textBoxScaleIncStep.Text = detector.ScaleIncrementStep.ToString();
        }
      }
    }

    private void buttonBrowseFace_Click(object sender, EventArgs e)
    {
      OpenFileDialog dlg = new OpenFileDialog();
      dlg.Filter = "txt files (*.txt)|*.txt|All files (*.*)|*.*";
      if (dlg.ShowDialog() == DialogResult.OK)
      {
        textBoxFaceInputFile.Text = dlg.FileName;
      }
    }

    private void buttonBrowseNonFace_Click(object sender, EventArgs e)
    {
      OpenFileDialog dlg = new OpenFileDialog();
      dlg.Filter = "txt files (*.txt)|*.txt|All files (*.*)|*.*";
      if (dlg.ShowDialog() == DialogResult.OK)
      {
        textBoxNonFaceInputFile.Text = dlg.FileName;
      }
    }

    private void buttonTraining_Click(object sender, EventArgs e)
    {
      Cursor = Cursors.WaitCursor;

      var featureExtractor = new HaarFaceFeatureExtractor();
      FeatureVectorSet trainingSet = new FeatureVectorSet();
      FeatureVector f;
      Image<byte> image;
      if (textBoxFaceInputFile.Text != "" && textBoxNonFaceInputFile.Text != "")
      {
        string[] faceImages = File.ReadAllLines(textBoxFaceInputFile.Text);
        string[] nonFaceImages = File.ReadAllLines(textBoxNonFaceInputFile.Text);
        string facePath = textBoxFaceInputFile.Text.Substring(0, textBoxFaceInputFile.Text.LastIndexOf('\\'));
        string nonFacePath = textBoxNonFaceInputFile.Text.Substring(0, textBoxNonFaceInputFile.Text.LastIndexOf('\\'));

        // Face
        foreach (string fileName in faceImages)
        {
          image = ImageBase.LoadFromFile(facePath + "\\" + fileName).ToByteImage();
          f = featureExtractor.ExtractFeatures(image);
          f.Class = 1;
          trainingSet.Add(f, false);
        }
        // Non face
        foreach (string fileName in nonFaceImages)
        {
          image = ImageBase.LoadFromFile(nonFacePath + "\\" + fileName).ToByteImage();
          f = featureExtractor.ExtractFeatures(image);
          f.Class = 0;
          trainingSet.Add(f, false);
        }

        SvmClassifierBuilder svmBuilder = new SvmClassifierBuilder(trainingSet);
        svmBuilder.SvmType = SvmTypes.C_SVC;
        svmBuilder.Cost = 10;
        svmBuilder.KernelType = KernelType.Polynomial;
        svmBuilder.Degree = 2;
        svmBuilder.Gamma = 0.1;

        var classifier = svmBuilder.Train();
        detector = new ClassifierFaceDetector(featureExtractor, classifier);
        UpdateParams();
      }

      Cursor = Cursors.Default;
    }

    private void buttonPreview_Click(object sender, EventArgs e)
    {
      if (detector != null)
      {
        UpdateParams();
        var previewForm = new AlgorithmPreviewForm(detector);
        previewForm.ShowDialog();
        textBoxMinWindowSize.Text = detector.MinimumWindowSize.ToString();
        textBoxMaxScaleFactor.Text = detector.MaximumScaleFactor.ToString();
        textBoxScaleIncStep.Text = detector.ScaleIncrementStep.ToString();
      }
    }

    private void buttonDetection_Click(object sender, EventArgs e)
    {
      if (detector != null)
      {
        Cursor = Cursors.WaitCursor;

        UpdateParams();
        detector.Detect(imageViewerInputImage.Image.ToByteImage());
        imageViewerFaceCandidates.Image = detector.InputImage;
        drawResult = true;

        Cursor = Cursors.Default;
      }
    }

    private void fileListControlInputFiles_SelectedFileNameChanged(object sender, EventArgs e)
    {
      imageViewerInputImage.Image = ImageBase.LoadFromFile(fileListControlInputFiles.SelectedFileName).ToByteImage();
      imageViewerFaceCandidates.Image = null;
      detector.InputImage = imageViewerInputImage.Image.ToByteImage();
      drawResult = false;
    }

    private void imageViewerFaceCandidates_Paint(object sender, PaintEventArgs e)
    {
      base.OnPaint(e);
      imageViewerFaceCandidates.AdjustGraphicsToWorldUnits(e.Graphics);
      if (detector != null && drawResult)
      {
        Graphics gr = e.Graphics;
        if (detector.IntermediateCandidates != null && checkBoxIntCand.Checked)
        {
          Pen penCand = new Pen(Color.Red, 0);
          foreach (FaceLocationCandidate intermediateCandidate in detector.IntermediateCandidates)
          {
            gr.DrawRectangle(penCand, intermediateCandidate.Rectangle.ToRectangle());
          }
        }
        if (detector.Result != null && checkBoxRes.Checked)
        {
          Pen penRes = new Pen(Color.Green, 1);
          foreach (FaceLocationCandidate candidate in detector.Result)
          {
            gr.DrawRectangle(penRes, candidate.Rectangle.ToRectangle());
          }
        }
      }
    }

    private void checkBoxIntCand_CheckedChanged(object sender, EventArgs e)
    {
      imageViewerFaceCandidates.Invalidate();
    }

    private void checkBoxRes_CheckedChanged(object sender, EventArgs e)
    {
      imageViewerFaceCandidates.Invalidate();
    }
  }
}
